@@ -51,20 +51,12 @@
 #include "gnome-vfs-volume-monitor-daemon.h"
 #include "gnome-vfs-volume-monitor-private.h"
 
-#define PATH_GCONF_GNOME_VFS_STORAGE "/system/storage"
-
 #ifndef PATHNAME_MAX
 # define PATHNAME_MAX	1024
 #endif
 
 typedef struct {
 	GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon;
-	gboolean display_internal_hard_drives;
-	gboolean display_scsi_drives;
-	gboolean display_scsi_optical_drives;
-	gboolean display_external_drives;
-	gboolean display_drives_removable;
-	guint gconf_client_connection_id;
 } GnomeVFSHalUserData;
 
 typedef enum {
@@ -617,39 +609,8 @@ _hal_drive_policy_check (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 
 	g_return_val_if_fail (hal_userdata != NULL, ret);
 
-	if (!hal_userdata->display_external_drives) {
-		if (libhal_drive_is_hotpluggable (hal_drive))
-			goto out;
-	}
-
-	if (!hal_userdata->display_drives_removable) {
-		if (libhal_drive_uses_removable_media (hal_drive))
-			goto out;
-	}
-
-	if (!hal_userdata->display_internal_hard_drives) {
-		if ((libhal_drive_uses_removable_media (hal_drive) == FALSE) && 
-		    (libhal_drive_is_hotpluggable (hal_drive) == FALSE))
-			goto out;
-	}
-
-	if (!hal_userdata->display_scsi_drives) {
-		if (libhal_drive_get_bus (hal_drive) == LIBHAL_DRIVE_BUS_SCSI) {
-			if (hal_userdata->display_scsi_optical_drives) {
-				if (libhal_drive_get_type (hal_drive) == LIBHAL_DRIVE_TYPE_CDROM) {
-					/* we're safe */
-				} else {
-					goto out;
-				}
-			} else {
-				goto out;
-			}
-		}
-	}
-
 	ret = TRUE;
 
-out:
 	return ret;
 }
 
@@ -689,12 +650,25 @@ _hal_volume_policy_check (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 	if (!_hal_drive_policy_check (volume_monitor_daemon, hal_drive, hal_volume))
 		goto out;
 
-	/* needs to be a mountable filesystem OR audio disc OR blank disc */
+	/* needs to be a mountable filesystem OR contain crypto bits OR audio disc OR blank disc */
 	if (! ((libhal_volume_get_fsusage (hal_volume) == LIBHAL_VOLUME_USAGE_MOUNTABLE_FILESYSTEM) ||
+	       (libhal_volume_get_fsusage (hal_volume) == LIBHAL_VOLUME_USAGE_CRYPTO) ||
 	       libhal_volume_disc_has_audio (hal_volume) ||
 	       libhal_volume_disc_is_blank (hal_volume)))
 		goto out;
 
+	/* if we contain crypto bits, only show if our cleartext volume is not yet setup */
+	if (libhal_volume_get_fsusage (hal_volume) == LIBHAL_VOLUME_USAGE_CRYPTO) {
+		char *clear_udi;
+
+		clear_udi = libhal_volume_crypto_get_clear_volume_udi (volume_monitor_daemon->hal_ctx, hal_volume);
+		if (clear_udi != NULL) {
+			free (clear_udi);
+			goto out;
+		}
+	}
+
+
 	/* for volumes the vendor and/or sysadmin wants to be ignore (e.g. bootstrap HFS
 	 * partitions on the Mac, HP_RECOVERY partitions on HP systems etc.)
 	 */
@@ -845,12 +819,14 @@ _hal_add_drive_without_volumes (GnomeVFSVolumeMonitorDaemon *volume_monitor_daem
 
 	/* don't add if it's already there */
 	drive = _gnome_vfs_volume_monitor_find_drive_by_hal_udi (volume_monitor, libhal_drive_get_udi (hal_drive));
-	if (drive != NULL)
+	if (drive != NULL) {
 		goto out;
+	}
 
 	/* doesn't make sense for devices without removable storage */
-	if (!libhal_drive_uses_removable_media (hal_drive))
+	if (!libhal_drive_uses_removable_media (hal_drive)) {
 		goto out;
+	}
 	
 	drive = g_object_new (GNOME_VFS_TYPE_DRIVE, NULL);
 	drive->priv->activation_uri = g_strdup ("");
@@ -888,17 +864,28 @@ _hal_add_volume (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 	GnomeVFSHalUserData *hal_userdata;
 	char *name;
 	gboolean allowed_by_policy;
+	const char *backing_udi;
 
 	g_return_val_if_fail (hal_drive != NULL, FALSE);
 	g_return_val_if_fail (hal_volume != NULL, FALSE);
 
 	ret = FALSE;
+	backing_udi = NULL;
 
 	volume_monitor = GNOME_VFS_VOLUME_MONITOR (volume_monitor_daemon);
 	hal_userdata = (GnomeVFSHalUserData *) libhal_ctx_get_user_data (volume_monitor_daemon->hal_ctx);
 
 	allowed_by_policy = _hal_volume_policy_check (volume_monitor_daemon, hal_drive, hal_volume);
 
+#ifdef HAL_SHOW_DEBUG
+	g_debug ("entering _hal_add_volume for\n"
+		 "  drive udi '%s'\n"
+		 "  volume udi '%s'\n"
+		 "  allowd_by_policy %s",
+		 libhal_drive_get_udi (hal_drive), libhal_volume_get_udi (hal_volume),
+		 allowed_by_policy ? "yes" : "no");
+#endif
+
 	if (!allowed_by_policy) {
 		/* make sure to completey delete any existing drive/volume for policy changes if the 
 		 * user_visible flag differs... */
@@ -934,10 +921,6 @@ _hal_add_volume (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 	if ( _hal_volume_temp_udi (volume_monitor_daemon, hal_drive, hal_volume))
 		goto out;
 
-#ifdef HAL_SHOW_DEBUG
-	g_debug ("entering _hal_add_volume for\n  drive udi '%s'\n  volume udi '%s'\n",
-		 libhal_drive_get_udi (hal_drive), libhal_volume_get_udi (hal_volume));
-#endif
 
 	/* OK, check if we got a drive_without_volumes drive and delete that since we're going to add a
 	 * drive for added partitions */
@@ -949,6 +932,31 @@ _hal_add_volume (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 		_gnome_vfs_volume_monitor_disconnected (volume_monitor, drive);
 	}
 
+	if (!allowed_by_policy && 
+	    libhal_volume_get_fsusage (hal_volume) != LIBHAL_VOLUME_USAGE_MOUNTABLE_FILESYSTEM)
+		goto out;
+
+	/* If we're stemming from a crypto volume... then remove the
+	 * GnomeVFSDrive we added so users had a way to invoke
+	 * gnome-mount for asking for the pass-phrase...
+	 */
+	backing_udi = libhal_volume_crypto_get_backing_volume_udi (hal_volume);
+	if (backing_udi != NULL) {
+		GnomeVFSDrive *backing_drive;
+		
+		backing_drive = _gnome_vfs_volume_monitor_find_drive_by_hal_udi (volume_monitor, backing_udi);
+		if (backing_drive != NULL) {
+#ifdef HAL_SHOW_DEBUG
+			g_debug ("Removing GnomeVFSDrive for crypto device with path %s "
+				 "(got cleartext device at path %s)", 
+				 backing_drive->priv->device_path,
+				 libhal_volume_get_device_file (hal_volume));
+#endif
+			_gnome_vfs_volume_monitor_disconnected (volume_monitor, backing_drive);
+		}
+		
+	}
+
 	/* if we had a drive from here but where we weren't mounted, just use that drive since nothing actually
 	 * changed 
 	 */
@@ -984,6 +992,7 @@ _hal_add_volume (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 		drive->priv->volumes = NULL;
 		drive->priv->hal_udi = g_strdup (libhal_volume_get_udi (hal_volume));
 		drive->priv->hal_drive_udi = g_strdup (libhal_drive_get_udi (hal_drive));
+		drive->priv->hal_backing_crypto_volume_udi = g_strdup (backing_udi);
                 drive->priv->must_eject_at_unmount = libhal_drive_requires_eject (hal_drive);
 
 #ifdef HAL_SHOW_DEBUG
@@ -1046,6 +1055,7 @@ _hal_add_volume (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 	
 	ret = TRUE;
 out:
+
 	return ret;
 }
 
@@ -1105,6 +1115,10 @@ _hal_update_all (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 
 				}
 
+#ifdef HAL_SHOW_DEBUG
+				g_debug ("  added %d volumes", num_volumes_added);
+#endif
+
 				if (num_volumes_added == 0) {
 					/* if we didn't add any volumes show the drive_without_volumes drive */
 					_hal_add_drive_without_volumes (volume_monitor_daemon, drive);
@@ -1242,12 +1256,45 @@ _hal_device_removed (LibHalContext *hal_ctx, const char *udi)
 	}
 
 	if (drive != NULL) {
+		char *backing_udi;
+
 		if (hal_drive_udi == NULL)
 			hal_drive_udi = g_strdup (drive->priv->hal_drive_udi);
 #ifdef HAL_SHOW_DEBUG
 		g_debug ("Removing GnomeVFSDrive for device path %s", drive->priv->device_path);
 #endif
+
+		if (drive->priv->hal_backing_crypto_volume_udi != NULL)
+			backing_udi = g_strdup (drive->priv->hal_backing_crypto_volume_udi);
+		else
+			backing_udi = NULL;
+
+
 		_gnome_vfs_volume_monitor_disconnected (GNOME_VFS_VOLUME_MONITOR (volume_monitor_daemon), drive);
+
+		if (backing_udi != NULL) {
+			LibHalVolume *crypto_volume;
+
+#ifdef HAL_SHOW_DEBUG
+			g_debug ("Adding back GnomeVFSDrive for crypto volume");
+#endif
+			crypto_volume = libhal_volume_from_udi (volume_monitor_daemon->hal_ctx, backing_udi);
+			if (crypto_volume != NULL) {
+				LibHalDrive *crypto_drive;
+				
+				crypto_drive = libhal_drive_from_udi (
+					volume_monitor_daemon->hal_ctx,
+					libhal_volume_get_storage_device_udi (crypto_volume));
+				if (crypto_drive != NULL) {
+					_hal_add_volume (volume_monitor_daemon, crypto_drive, crypto_volume);
+					libhal_drive_free (crypto_drive);
+				}
+				libhal_volume_free (crypto_volume);
+			}
+			
+			g_free (backing_udi);
+		}
+
 	}
 
 #ifdef HAL_SHOW_DEBUG
@@ -1356,67 +1403,6 @@ out:
 	if (volume != NULL)
 		libhal_volume_free (volume);
 }
-	
-
-static void 
-_hal_get_settings_from_gconf (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
-{
-	GnomeVFSHalUserData *hal_userdata;
-
-	g_return_if_fail (volume_monitor_daemon != NULL);
-
-	hal_userdata = (GnomeVFSHalUserData *) libhal_ctx_get_user_data (volume_monitor_daemon->hal_ctx);
-
-	g_return_if_fail (hal_userdata != NULL);
-
-	/* respect the gconf settings! */
-	hal_userdata->display_internal_hard_drives = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
-									    PATH_GCONF_GNOME_VFS_STORAGE 
-									    "/display_internal_hard_drives",
-									    NULL);
-	hal_userdata->display_scsi_drives = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
-								   PATH_GCONF_GNOME_VFS_STORAGE 
-								   "/display_scsi_drives",
-								   NULL);
-	hal_userdata->display_scsi_optical_drives = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
-									   PATH_GCONF_GNOME_VFS_STORAGE 
-									   "/display_scsi_optical_drives",
-									   NULL);
-	hal_userdata->display_external_drives = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
-								       PATH_GCONF_GNOME_VFS_STORAGE
-								       "/display_external_drives",
-								       NULL);
-	hal_userdata->display_drives_removable = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
-									PATH_GCONF_GNOME_VFS_STORAGE
-									"/display_drives_with_removable_media",
-									NULL);
-}
-
-static void
-_hal_settings_changed (GConfClient* client,
-		       guint cnxn_id,
-		       GConfEntry *entry,
-		       gpointer data)
-{
-	GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon;
-
-	volume_monitor_daemon = data;
-
-	g_return_if_fail (volume_monitor_daemon != NULL);
-
-#ifdef HAL_SHOW_DEBUG
-	g_debug ("in _hal_settings_changed");
-#endif
-
-#if 0
-	/* TODO: presently we don't support reloading settings from gconf. This
-	 *       requires some fixes not presently implemented
-	 */
-	_hal_get_settings_from_gconf (volume_monitor_daemon);
-	_gnome_vfs_hal_mounts_force_reprobe (volume_monitor_daemon);
-#endif
-
-}
 
 
 gboolean
@@ -1469,27 +1455,12 @@ _gnome_vfs_hal_mounts_init (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 	libhal_ctx_set_user_data (volume_monitor_daemon->hal_ctx,
 				  hal_userdata);
 
-	gconf_client_add_dir (volume_monitor_daemon->gconf_client,
-			      PATH_GCONF_GNOME_VFS_STORAGE,
-			      GCONF_CLIENT_PRELOAD_RECURSIVE,
-			      NULL);
-
-	hal_userdata->gconf_client_connection_id =
-		gconf_client_notify_add (volume_monitor_daemon->gconf_client,
-					 PATH_GCONF_GNOME_VFS_STORAGE,
-					 _hal_settings_changed,
-					 volume_monitor_daemon,
-					 NULL,
-					 NULL);
-
 	/* Simply watch all property changes instead of dynamically
 	 * adding/removing match rules bus-side to only match certain
 	 * objects...
 	 */
 	libhal_device_property_watch_all (volume_monitor_daemon->hal_ctx, &error);
 
-	_hal_get_settings_from_gconf (volume_monitor_daemon);
-
 	/* add drives/volumes from HAL */
 	_hal_update_all (volume_monitor_daemon);
 
@@ -1515,9 +1486,6 @@ _gnome_vfs_hal_mounts_shutdown (GnomeVFSVolumeMonitorDaemon *volume_monitor_daem
 		g_warning ("hal_shutdown failed - unable to free hal context\n");
 	}
 
-	gconf_client_notify_remove (volume_monitor_daemon->gconf_client,
-				    hal_userdata->gconf_client_connection_id);
-
 }
 
 void
