@@ -26,7 +26,9 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/time.h>
+#include <time.h>
 #include <glib.h>
+#include <glib/gstdio.h>
 #include "gnome-vfs-mime.h"
 #include "gnome-vfs-mime-info-cache.h"
 #include "gnome-vfs-mime-info.h"
@@ -34,6 +36,7 @@
 #include "gnome-vfs-monitor.h"
 #include "gnome-vfs-ops.h"
 #include "gnome-vfs-utils.h"
+#include "xdgmime.h"
 
 typedef struct {
 	char *path;
@@ -54,6 +57,7 @@ typedef struct {
 } GnomeVFSMimeInfoCache;
 
 extern void _gnome_vfs_mime_monitor_emit_data_changed (GnomeVFSMIMEMonitor *monitor); 
+extern void _gnome_vfs_mime_info_cache_init (void);
 
 static void gnome_vfs_mime_info_cache_dir_init (GnomeVFSMimeInfoCacheDir *dir);
 static void gnome_vfs_mime_info_cache_dir_init_defaults_list (GnomeVFSMimeInfoCacheDir *dir);
@@ -66,7 +70,6 @@ static gboolean gnome_vfs_mime_info_cache_dir_desktop_entry_is_valid (GnomeVFSMi
 static void gnome_vfs_mime_info_cache_dir_add_desktop_entries (GnomeVFSMimeInfoCacheDir *dir,
 							       const char *mime_type,
 							       char **new_desktop_file_ids);
-static void gnome_vfs_mime_info_cache_init (void);
 static GnomeVFSMimeInfoCache *gnome_vfs_mime_info_cache_new (void);
 static void gnome_vfs_mime_info_cache_free (GnomeVFSMimeInfoCache *cache);
 
@@ -97,7 +100,7 @@ gnome_vfs_mime_info_cache_dir_out_of_date (GnomeVFSMimeInfoCacheDir *dir, const
 
 	filename = g_build_filename (dir->path, cache_file, NULL);
 
-	if (stat (filename, &buf) < 0) {
+	if (g_stat (filename, &buf) < 0) {
 		g_free (filename);
 		return TRUE;
 	}
@@ -155,7 +158,7 @@ gnome_vfs_mime_info_cache_dir_init (GnomeVFSMimeInfoCacheDir *dir)
 
 	filename = g_build_filename (dir->path, "mimeinfo.cache", NULL);
 
-	if (stat (filename, &buf) < 0)
+	if (g_stat (filename, &buf) < 0)
 		goto error;
 
 	if (dir->mime_info_cache_timestamp > 0) 
@@ -245,7 +248,7 @@ gnome_vfs_mime_info_cache_dir_init_defaults_list (GnomeVFSMimeInfoCacheDir *dir)
 
 	filename = g_build_filename (dir->path, "defaults.list", NULL);
 
-	if (stat (filename, &buf) < 0)
+	if (g_stat (filename, &buf) < 0)
 		goto error;
 
 	if (dir->defaults_list_timestamp > 0) 
@@ -301,6 +304,13 @@ error:
 		g_error_free (load_error);
 }
 
+static gboolean
+emit_mime_changed (gpointer data)
+{
+	_gnome_vfs_mime_monitor_emit_data_changed (gnome_vfs_mime_monitor_get ());
+	return FALSE;
+}
+
 static void
 gnome_vfs_mime_info_cache_dir_changed (GnomeVFSMonitorHandle    *handle,
 		                       const gchar              *monitor_uri,
@@ -311,7 +321,9 @@ gnome_vfs_mime_info_cache_dir_changed (GnomeVFSMonitorHandle    *handle,
 	G_LOCK (mime_info_cache);
 	gnome_vfs_mime_info_cache_blow_global_cache ();
 	gnome_vfs_mime_info_cache_dir_init (dir);
+	mime_info_cache->should_ping_mime_monitor = FALSE;
 	G_UNLOCK (mime_info_cache);
+	g_idle_add (emit_mime_changed, NULL);
 }
 
 static void
@@ -324,7 +336,9 @@ gnome_vfs_mime_info_cache_dir_defaults_changed (GnomeVFSMonitorHandle    *handle
 	G_LOCK (mime_info_cache);
 	gnome_vfs_mime_info_cache_blow_global_cache ();
 	gnome_vfs_mime_info_cache_dir_init_defaults_list (dir);
+	mime_info_cache->should_ping_mime_monitor = FALSE;
 	G_UNLOCK (mime_info_cache);
+	g_idle_add (emit_mime_changed, NULL);
 }
 
 static GnomeVFSMimeInfoCacheDir *
@@ -437,6 +451,7 @@ gnome_vfs_mime_info_cache_dir_desktop_entry_is_valid (GnomeVFSMimeInfoCacheDir *
 
 	if (load_error != NULL) {
 		g_error_free (load_error);
+		g_key_file_free (key_file);
 		return FALSE;
 	}
 
@@ -480,6 +495,8 @@ gnome_vfs_mime_info_cache_dir_desktop_entry_is_valid (GnomeVFSMimeInfoCacheDir *
 
 		if (load_error != NULL) {
 			g_error_free (load_error);
+			g_strfreev (not_show_in_list);
+			g_key_file_free (key_file);
 			return FALSE;
 		}
 
@@ -585,28 +602,20 @@ gnome_vfs_mime_info_cache_update_dir_lists (void)
 	}
 }
 
-static gboolean
-emit_mime_changed (gpointer data)
-{
-	_gnome_vfs_mime_monitor_emit_data_changed (gnome_vfs_mime_monitor_get ());
-	return FALSE;
-}
-
-
-static void
-gnome_vfs_mime_info_cache_init (void)
+void
+_gnome_vfs_mime_info_cache_init (void)
 {
 	G_LOCK (mime_info_cache);
 	if (mime_info_cache == NULL) {
 		gnome_vfs_mime_info_cache_init_dir_lists ();
 	} else {
-		struct timeval tv;
+		time_t now;
 
-		gettimeofday (&tv, NULL);
+		time (&now);
 
-		if (tv.tv_sec >= mime_info_cache->last_stat_time + 5) {
+		if (now >= mime_info_cache->last_stat_time + 5) {
 			gnome_vfs_mime_info_cache_update_dir_lists ();
-			mime_info_cache->last_stat_time = tv.tv_sec;
+			mime_info_cache->last_stat_time = now;
 		}
 	}
 
@@ -645,6 +654,12 @@ gnome_vfs_mime_info_cache_free (GnomeVFSMimeInfoCache *cache)
 	g_free (cache);
 }
 
+/**
+ * gnome_vfs_mime_info_cache_reload:
+ * @dir: directory path which needs reloading.
+ * 
+ * Reload the mime information for the @dir.
+ */
 void
 gnome_vfs_mime_info_cache_reload (const char *dir)
 {
@@ -659,121 +674,78 @@ gnome_vfs_mime_info_cache_reload (const char *dir)
 	}
 }
 
-
-
-
-static void
-get_all_parent_types_helper (GList **mime_types, const char *mime_type)
-{
-	const gchar *parent_list;
-	
-	if (!g_list_find_custom (*mime_types, mime_type,
-				 (GCompareFunc) strcmp)) {
-		*mime_types = g_list_prepend (*mime_types, g_strdup (mime_type));
-	}
-
-	parent_list = gnome_vfs_mime_get_value (mime_type, "parent_classes");
-	if (parent_list) {
-		char **parents;
-		int i;
-		
-		parents = g_strsplit (parent_list, ":", -1);
-		for (i = 0; parents && parents[i] != NULL; i++) {
-			get_all_parent_types_helper (mime_types, parents[i]);
-		}
-		g_strfreev (parents);
-	}
-
-	if (g_str_has_prefix (mime_type, "text/")) {
-		if (!g_list_find_custom (*mime_types, "text/plain",
-					 (GCompareFunc) strcmp)) {
-			*mime_types = g_list_prepend (*mime_types, g_strdup ("text/plain"));
-		}
-	}
-	if (!g_str_has_prefix (mime_type, "inode/")) {
-		if (!g_list_find_custom (*mime_types, "application/octet-stream",
-					 (GCompareFunc) strcmp)) {
-			*mime_types = g_list_prepend (*mime_types, g_strdup ("application/octet-stream"));
-		}
-	}
-}
-
 static GList *
 get_all_parent_types (const char *mime_type)
 {
-	GList *mime_types;
+	const char *umime;
+	const char **parents;
+	GList *l = NULL;
+	int i;
 
-	/* TODO: Unalias mime_type first */
-	
-	mime_types = NULL;
-	get_all_parent_types_helper (&mime_types, mime_type);
-	return g_list_reverse (mime_types);
-}
+	umime = xdg_mime_unalias_mime_type (mime_type);
+	l = g_list_prepend (l, g_strdup (umime));
 
-GList *
-gnome_vfs_mime_get_all_desktop_entries (const char *mime_type)
-{
-	GList *desktop_entries, *list, *dir_list, *tmp;
-	GList *mime_types, *m_list;
-	GnomeVFSMimeInfoCacheDir *dir;
-	char *type;
+	parents = xdg_mime_get_mime_parents (mime_type);
 
-	gnome_vfs_mime_info_cache_init ();
-
-	G_LOCK (mime_info_cache);
-	mime_types = get_all_parent_types (mime_type);
-
-	desktop_entries = NULL;
-	for (m_list = mime_types; m_list != NULL; m_list = m_list->next) {
-		type = m_list->data;
-		
-		for (dir_list = mime_info_cache->dirs;
-		     dir_list != NULL;
-		     dir_list = dir_list->next) {
-			dir = (GnomeVFSMimeInfoCacheDir *) dir_list->data;
-			
-			list = g_hash_table_lookup (dir->mime_info_cache_map, type);
-			
-			for (tmp = list; tmp != NULL; tmp = tmp->next) {
-				/* Add if not already in list, and valid */
-				/* TODO: Should we cache here to avoid desktop file validation? */
-				if (!g_list_find_custom (desktop_entries, tmp->data,
-							 (GCompareFunc) strcmp) &&
-				    gnome_vfs_mime_info_desktop_entry_is_valid (tmp->data)) {
-					desktop_entries = g_list_prepend (desktop_entries,
-									  g_strdup (tmp->data));
-				}
-			}
-		}
+	for (i = 0; parents && parents[i] != NULL; i++) {
+		l = g_list_prepend (l, g_strdup (parents[i]));
 	}
 
-	G_UNLOCK (mime_info_cache);
-
-	desktop_entries = g_list_reverse (desktop_entries);
-
-	return desktop_entries;
+	return g_list_reverse (l);
 }
 
-gchar *
-gnome_vfs_mime_get_default_desktop_entry (const char *mime_type)
+static GList *
+append_desktop_entry (GList *list, const char *desktop_entry)
+{
+	/* Add if not already in list, and valid */
+	/* TODO: Should we cache here to avoid desktop file validation? */
+	if (!g_list_find_custom (list, desktop_entry, (GCompareFunc) strcmp) &&
+	    gnome_vfs_mime_info_desktop_entry_is_valid (desktop_entry)) {
+		list = g_list_prepend (list, g_strdup (desktop_entry));
+	}
+	return list;
+}
+
+static gchar *
+get_default_desktop_entry (const char *mime_type)
 {
 	gchar *desktop_entry;
 	char **desktop_entries;
-	GList *dir_list;
+	GList *dir_list, *l;
+	GList *folder_handlers;
 	GnomeVFSMimeInfoCacheDir *dir;
 	int i;
 
-	gnome_vfs_mime_info_cache_init ();
-
-	G_LOCK (mime_info_cache);
-
 	desktop_entry = g_hash_table_lookup (mime_info_cache->global_defaults_cache,
 					     mime_type);
 	if (desktop_entry) {
-		G_UNLOCK (mime_info_cache);
 		return g_strdup (desktop_entry);
 	}
 
+	if (g_str_has_prefix (mime_type, "x-directory/")) {
+		/* We special case folder handler defaults so that
+		 * we always can get a working handler for them
+		 * even though we don't control the defaults.list,
+		 * or the list of availible directory handlers.
+		 */
+
+		for (dir_list = mime_info_cache->dirs; dir_list != NULL; dir_list = dir_list->next) {
+			dir = dir_list->data;
+		
+			folder_handlers = g_hash_table_lookup (dir->mime_info_cache_map, "x-directory/gnome-default-handler");
+			for (l = folder_handlers; l != NULL; l = l->next) {
+				desktop_entry = l->data;
+			
+				if (desktop_entry != NULL &&
+				    gnome_vfs_mime_info_desktop_entry_is_valid (desktop_entry)) {
+					g_hash_table_insert (mime_info_cache->global_defaults_cache,
+							     g_strdup (mime_type), g_strdup (desktop_entry));
+					return g_strdup (desktop_entry);
+				}
+			}
+		}
+	}
+	
 	desktop_entry = NULL;
 	for (dir_list = mime_info_cache->dirs; dir_list != NULL; dir_list = dir_list->next) {
 		dir = dir_list->data;
@@ -785,13 +757,98 @@ gnome_vfs_mime_get_default_desktop_entry (const char *mime_type)
 			    gnome_vfs_mime_info_desktop_entry_is_valid (desktop_entry)) {
 				g_hash_table_insert (mime_info_cache->global_defaults_cache,
 						     g_strdup (mime_type), g_strdup (desktop_entry));
-				G_UNLOCK (mime_info_cache);
 				return g_strdup (desktop_entry);
 			}
 		}
 	}
-	G_UNLOCK (mime_info_cache);
 
 	return NULL;
 }
 
+/**
+ * gnome_vfs_mime_get_all_desktop_entries:
+ * @mime_type: a mime type.
+ *
+ * Returns all the desktop filenames for @mime_type. The desktop files
+ * are listed in an order so that default applications are listed before
+ * non-default ones, and handlers for inherited mimetypes are listed
+ * after the base ones.
+ *
+ * Return value: a #GList containing the desktop filenames containing the
+ * @mime_type.
+ */
+GList *
+gnome_vfs_mime_get_all_desktop_entries (const char *base_mime_type)
+{
+	GList *desktop_entries, *list, *dir_list, *tmp;
+	GList *mime_types, *m_list;
+	GnomeVFSMimeInfoCacheDir *dir;
+	char *mime_type;
+	char *default_desktop_entry;
+
+	_gnome_vfs_mime_info_cache_init ();
+
+	G_LOCK (mime_info_cache);
+	mime_types = get_all_parent_types (base_mime_type);
+
+	desktop_entries = NULL;
+	for (m_list = mime_types; m_list != NULL; m_list = m_list->next) {
+		mime_type = m_list->data;
+
+		default_desktop_entry = get_default_desktop_entry (mime_type);
+		if (default_desktop_entry) {
+			desktop_entries = append_desktop_entry (desktop_entries, default_desktop_entry);
+			g_free (default_desktop_entry);
+		}
+		
+		for (dir_list = mime_info_cache->dirs;
+		     dir_list != NULL;
+		     dir_list = dir_list->next) {
+			dir = (GnomeVFSMimeInfoCacheDir *) dir_list->data;
+			
+			list = g_hash_table_lookup (dir->mime_info_cache_map, mime_type);
+			
+			for (tmp = list; tmp != NULL; tmp = tmp->next) {
+				desktop_entries = append_desktop_entry (desktop_entries, tmp->data);
+			}
+		}
+	}
+
+	G_UNLOCK (mime_info_cache);
+
+	g_list_foreach (mime_types, (GFunc)g_free, NULL);
+	g_list_free (mime_types);
+
+	desktop_entries = g_list_reverse (desktop_entries);
+
+	return desktop_entries;
+}
+
+
+/**
+ * gnome_vfs_mime_get_default_desktop_entry:
+ * @mime_type: a mime type.
+ *
+ * Returns the default desktop filename for @mime_type.
+ *
+ * Return value: the default desktop filename for @mime_type.
+ */
+gchar *
+gnome_vfs_mime_get_default_desktop_entry (const char *mime_type)
+{
+	char *desktop_entry;
+	GList *desktop_files;
+
+	desktop_entry = NULL;
+	
+	desktop_files =	gnome_vfs_mime_get_all_desktop_entries (mime_type);
+	if (desktop_files != NULL) {
+		desktop_entry = g_strdup (desktop_files->data);
+	}
+		
+	g_list_foreach (desktop_files, (GFunc)g_free, NULL);
+	g_list_free (desktop_files);
+
+	return desktop_entry;
+}
+
