@@ -491,11 +491,15 @@ parse_uri_substring (const gchar *substring, GnomeVFSURI *parent)
 
 /**
  * gnome_vfs_uri_new:
- * @text_uri: a string representing a uri.
+ * @text_uri: an escaped string representing a uri.
  * 
  * Create a new uri from @text_uri. Unsupported and unsafe methods
  * are not allowed and will result in %NULL being returned. URL
  * transforms are allowed.
+ *
+ * The @a text_uri must be an escaped URI string such as returned by 
+ * gnome_vfs_get_uri_from_local_path(), gnome_vfs_make_uri_from_input(), 
+ * or gtk_file_chooser_get_uri().
  * 
  * Return value: The new uri.
  */
@@ -909,7 +913,7 @@ gnome_vfs_uri_resolve_relative (const GnomeVFSURI *base,
 	char *text_new;
 	GnomeVFSURI *uri;
 
-	g_assert (relative_reference != NULL);
+	g_return_val_if_fail (relative_reference != NULL, NULL);
 
 	if (base == NULL) {
 		text_base = g_strdup ("");
@@ -1107,7 +1111,10 @@ gnome_vfs_uri_append_path (const GnomeVFSURI *uri,
 {
 	gchar *escaped_string;
 	GnomeVFSURI *new_uri;
-	
+
+	g_return_val_if_fail (uri != NULL, NULL);
+	g_return_val_if_fail (path != NULL, NULL);
+
 	escaped_string = gnome_vfs_escape_path_string (path);
 	new_uri = gnome_vfs_uri_append_string (uri, escaped_string);
 	g_free (escaped_string);
@@ -1117,11 +1124,12 @@ gnome_vfs_uri_append_path (const GnomeVFSURI *uri,
 /**
  * gnome_vfs_uri_append_file_name:
  * @uri: a #GnomeVFSURI.
- * @filename: any "regular" file name (can include #, /, etc).
+ * @filename: any "regular" file name (can include #, /, etc) in the file system encoding. This is not an escaped URI.
  * 
  * Create a new uri obtained by appending @file_name to @uri.  This will take care
  * of adding an appropriate directory separator between the end of @uri and
- * the start of @file_name if necessary.
+ * the start of @file_name if necessary. @file_name might, for instance, be the 
+ * result of a call to g_dir_read_name().
  * 
  * Return value: The new uri obtained by combining @uri and @path.
  */
@@ -1131,6 +1139,9 @@ gnome_vfs_uri_append_file_name (const GnomeVFSURI *uri,
 {
 	gchar *escaped_string;
 	GnomeVFSURI *new_uri;
+
+	g_return_val_if_fail (uri != NULL, NULL);
+	g_return_val_if_fail (filename != NULL, NULL);
 	
 	escaped_string = gnome_vfs_escape_string (filename);
 	new_uri = gnome_vfs_uri_append_string (uri, escaped_string);
@@ -1167,6 +1178,8 @@ gnome_vfs_uri_to_string (const GnomeVFSURI *uri,
 	GString *string;
 	gchar *result, *tmp;
 
+	g_return_val_if_fail (uri != NULL, NULL);
+
 	string = g_string_new (uri->method_string);
 	g_string_append_c (string, ':');
 
@@ -1272,9 +1285,12 @@ gnome_vfs_uri_to_string (const GnomeVFSURI *uri,
  * gnome_vfs_uri_is_local:
  * @uri: a #GnomeVFSURI.
  * 
- * Check if @uri is a local (native) file system.
+ * Check if @uri is a local URI. Note that the return value of this
+ * function entirely depends on the #GnomeVFSMethod associated with the
+ * URI. It is up to the method author to distinguish between remote URIs
+ * add URIs referring to entities on the local computer.
  * 
- * Return value: %FALSE if @uri is not a local file system, %TRUE otherwise.
+ * Return value: %TRUE if @uri refers to a local entity, %FALSE otherwise.
  */
 gboolean
 gnome_vfs_uri_is_local (const GnomeVFSURI *uri)
@@ -1303,6 +1319,8 @@ gnome_vfs_uri_has_parent (const GnomeVFSURI *uri)
 {
 	GnomeVFSURI *parent;
 
+	g_return_val_if_fail (uri != NULL, FALSE);
+
 	parent = gnome_vfs_uri_get_parent (uri);
 	if (parent == NULL) {
 		return FALSE;
@@ -1426,6 +1444,8 @@ gnome_vfs_uri_get_host_name (const GnomeVFSURI *uri)
 const gchar *
 gnome_vfs_uri_get_scheme (const GnomeVFSURI *uri)
 {
+	g_return_val_if_fail (uri != NULL, NULL);
+
 	return uri->method_string;
 }
 
@@ -1695,6 +1715,9 @@ gnome_vfs_uri_is_parent (const GnomeVFSURI *possible_parent,
 	GnomeVFSURI *item_parent_uri;
 	GnomeVFSURI *item;
 
+	g_return_val_if_fail (possible_parent != NULL, FALSE);
+	g_return_val_if_fail (possible_child != NULL, FALSE);
+
 	if (!recursive) {
 		item_parent_uri = gnome_vfs_uri_get_parent (possible_child);
 
@@ -1821,6 +1844,8 @@ gnome_vfs_uri_extract_short_name (const GnomeVFSURI *uri)
 	gchar *escaped_short_path_name, *short_path_name;
 	const gchar *host_name;
 
+	g_return_val_if_fail (uri != NULL, NULL);
+
 	escaped_short_path_name = gnome_vfs_uri_extract_short_path_name (uri);
 	short_path_name = gnome_vfs_unescape_string (escaped_short_path_name, "/");
 	g_free (escaped_short_path_name);
